#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = __importDefault(require("commander"));
// eslint-disable-next-line @typescript-eslint/no-var-requires
const { version, bin } = require("../package.json");
const name = Object.keys(bin)[0];
const ValidationService_1 = require("./ValidationService");
const core_1 = require("@ethereum-sourcify/core");
__exportStar(require("./ValidationService"), exports);
const NAME_DELIMITER = ":";
function parseRecompilationTarget(input) {
    const delimiterIndex = input.indexOf(NAME_DELIMITER);
    if (delimiterIndex === -1) {
        return input;
    }
    return { path: input.slice(0, delimiterIndex), name: input.slice(delimiterIndex + 1) };
}
function matches(contract, recompilationTarget) {
    if (!recompilationTarget) {
        return false;
    }
    if (typeof recompilationTarget === "string") {
        if (recompilationTarget === contract.name || recompilationTarget === contract.compiledPath) {
            return true;
        }
        return false;
    }
    const nameMatches = recompilationTarget.name === contract.name;
    const pathMatches = recompilationTarget.path === contract.compiledPath;
    return nameMatches && pathMatches;
}
function logOutput(recompilableContracts, programOptions) {
    switch (recompilableContracts.length) {
        case 0: {
            console.log(`Could not find the ${OPTION_NAME} target: ${programOptions.prepareJson}`);
            process.exitCode = 1;
            break;
        }
        case 1: {
            const standardJson = recompilableContracts[0].getStandardJson(programOptions.settings);
            const prettifiedStandardJson = JSON.stringify(standardJson, null, programOptions.pretty ? 2 : undefined);
            console.log(prettifiedStandardJson);
            break;
        }
        default: {
            console.log(`Ambiguous ${OPTION_NAME} target: ${programOptions.prepareJson}`);
            console.log(`${recompilableContracts.length} instances encountered.`);
            process.exitCode = 1;
            break;
        }
    }
}
const OPTION_NAME = "prepare-json";
commander_1.default
    .name(name)
    .helpOption("-h, --help", "Output the help message.")
    .version(version, "-v, --version", "Output the current version.")
    .description("A tool that validates smart contracts contained in the provided files, directories and zip archives.")
    .usage("[options] <path>...")
    .option(`-j, --${OPTION_NAME} <path:name>`, "Output only a JSON object to be used as standard-json input to solc.\n" +
    "The expected argument ought to be of the form path:name; ommiting any of the two is tolerated if unambiguous.\n" +
    `E.g. \`${name} --${OPTION_NAME} foo/bar/Contract.sol:ContractName path/to/dir path/to/zip\`\n`)
    .option("-S, --no-settings", "Omit the original settings when preparing a standard-json.\n" +
    `Only available with the --${OPTION_NAME} option\n`, true)
    .option("-p, --pretty", "Output the standard json in a pretty format.\n" +
    `Only available with the --${OPTION_NAME} option\n`);
if (require.main === module) {
    commander_1.default.parse(process.argv);
    const fileNames = commander_1.default.args; // what's left from process.argv
    const fileNamesJoint = fileNames.join(" ");
    if (!fileNames.length) {
        commander_1.default.help();
    }
    const recompilationTarget = commander_1.default.prepareJson ? parseRecompilationTarget(commander_1.default.prepareJson) : commander_1.default.prepareJson;
    const validationService = new ValidationService_1.ValidationService();
    let checkedContracts = [];
    const ignoring = [];
    try {
        checkedContracts = validationService.checkPaths(fileNames, ignoring);
    }
    catch (err) {
        console.log(err.message);
        process.exitCode = 1;
    }
    const recompilableContracts = [];
    const contractMessages = [];
    let invalidContracts = 0;
    checkedContracts.forEach(contract => {
        if (matches(contract, recompilationTarget)) {
            if (core_1.CheckedContract.isValid(contract)) {
                recompilableContracts.push(contract);
            }
            else {
                console.log(contract.getInfo());
                console.log(`The ${OPTION_NAME} target is not a valid contract! Exiting.`);
                process.exit(1);
            }
        }
        else if (recompilationTarget === undefined) {
            let msg = contract.getInfo(); // should be without a trailing newline
            if (core_1.CheckedContract.isValid(contract)) {
                msg += "\n";
                msg += `  To recompile, use: \`${name} --${OPTION_NAME} ${contract.compiledPath}:${contract.name} ${fileNamesJoint}`;
                msg += ` | solc --standard-json\``;
            }
            else {
                invalidContracts++;
            }
            contractMessages.push(msg);
        }
    });
    if (contractMessages.length) {
        console.log(contractMessages.join("\n\n"));
    }
    if (recompilationTarget !== undefined) {
        logOutput(recompilableContracts, commander_1.default.opts());
    }
    else if (ignoring.length) {
        console.log("\nFiles ignored due to readability issues:");
        ignoring.forEach(ignored => console.log("  " + ignored));
    }
    if (invalidContracts > 0) {
        process.exitCode = 1;
    }
}
//# sourceMappingURL=index.js.map